@@ -22,6 +22,7 @@
 #include "karchive.h"
 #include "karchive_p.h"
 #include "klimitediodevice_p.h"
+#include "loggingcategory.h"
 
 #include <qplatformdefs.h> // QT_STATBUF, QT_LSTAT
 
@@ -57,7 +58,7 @@ KArchive::KArchive(const QString &fileName)
     : d(new KArchivePrivate)
 {
     if (fileName.isEmpty()) {
-        qWarning("KArchive: No file name specified");
+        qCWarning(KArchiveLog) << "KArchive: No file name specified";
     }
     d->fileName = fileName;
     // This constructor leaves the device set to 0.
@@ -68,7 +69,7 @@ KArchive::KArchive(QIODevice *dev)
     : d(new KArchivePrivate)
 {
     if (!dev) {
-        qWarning("KArchive: Null device specified");
+        qCWarning(KArchiveLog) << "KArchive: Null device specified";
     }
     d->dev = dev;
 }
@@ -95,17 +96,19 @@ bool KArchive::open(QIODevice::OpenMode mode)
     }
 
     if (!d->dev) {
+        setErrorString(tr("No filename or device was specified"));
         return false;
     }
 
     if (!d->dev->isOpen() && !d->dev->open(mode)) {
+        setErrorString(tr("Could not set device mode to %1").arg(mode));
         return false;
     }
 
     d->mode = mode;
 
     Q_ASSERT(!d->rootDir);
-    d->rootDir = 0;
+    d->rootDir = nullptr;
 
     return openArchive(mode);
 }
@@ -116,12 +119,16 @@ bool KArchive::createDevice(QIODevice::OpenMode mode)
     case QIODevice::WriteOnly:
         if (!d->fileName.isEmpty()) {
             // The use of QSaveFile can't be done in the ctor (no mode known yet)
-            //qDebug() << "Writing to a file using QSaveFile";
+            //qCDebug(KArchiveLog) << "Writing to a file using QSaveFile";
             d->saveFile = new QSaveFile(d->fileName);
             if (!d->saveFile->open(QIODevice::WriteOnly)) {
-                //qWarning() << "QSaveFile creation for " << d->fileName << " failed, " << d->saveFile->errorString();
+                setErrorString(
+                    tr("QSaveFile creation for %1 failed: %2")
+                        .arg(d->fileName)
+                        .arg(d->saveFile->errorString()));
+
                 delete d->saveFile;
-                d->saveFile = 0;
+                d->saveFile = nullptr;
                 return false;
             }
             d->dev = d->saveFile;
@@ -137,7 +144,7 @@ bool KArchive::createDevice(QIODevice::OpenMode mode)
         }
         break; // continued below
     default:
-        //qWarning() << "Unsupported mode " << d->mode;
+        setErrorString(tr("Unsupported mode %1").arg(d->mode));
         return false;
     }
     return true;
@@ -146,6 +153,7 @@ bool KArchive::createDevice(QIODevice::OpenMode mode)
 bool KArchive::close()
 {
     if (!isOpen()) {
+        setErrorString(tr("Archive already closed"));
         return false;    // already closed (return false or true? arguable...)
     }
 
@@ -168,19 +176,24 @@ bool KArchive::close()
     if (d->saveFile) {
         closeSucceeded = d->saveFile->commit();
         delete d->saveFile;
-        d->saveFile = 0;
+        d->saveFile = nullptr;
     }
     if (d->deviceOwned) {
         delete d->dev; // we created it ourselves in open()
     }
 
     delete d->rootDir;
-    d->rootDir = 0;
+    d->rootDir = nullptr;
     d->mode = QIODevice::NotOpen;
-    d->dev = 0;
+    d->dev = nullptr;
     return closeSucceeded;
 }
 
+QString KArchive::errorString() const
+{
+    return d->errorStr;
+}
+
 const KArchiveDirectory *KArchive::directory() const
 {
     // rootDir isn't const so that parsing-on-demand is possible
@@ -191,7 +204,9 @@ bool KArchive::addLocalFile(const QString &fileName, const QString &destName)
 {
     QFileInfo fileInfo(fileName);
     if (!fileInfo.isFile() && !fileInfo.isSymLink()) {
-        //qWarning() << fileName << "doesn't exist or is not a regular file.";
+        setErrorString(
+            tr("%1 doesn't exist or is not a regular file.")
+            .arg(fileName));
         return false;
     }
 
@@ -202,8 +217,10 @@ bool KArchive::addLocalFile(const QString &fileName, const QString &destName)
 #endif
     QT_STATBUF fi;
     if (STAT_METHOD(QFile::encodeName(fileName).constData(), &fi) == -1) {
-        /*qWarning() << "stat'ing" << fileName
-            << "failed:" << strerror(errno);*/
+        setErrorString(
+            tr("Failed accessing the file %1 for adding to the archive. The error was: %2")
+            .arg(fileName)
+            .arg(QLatin1Literal{strerror(errno)}));
         return false;
     }
 
@@ -244,13 +261,16 @@ bool KArchive::addLocalFile(const QString &fileName, const QString &destName)
     // header and the tar file is effectively f*cked up
     QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly)) {
-        //qWarning() << "couldn't open file " << fileName;
+        setErrorString(
+            tr("Couldn't open file %1: %2")
+            .arg(fileName)
+            .arg(file.errorString()));
         return false;
     }
 
     if (!prepareWriting(destName, fileInfo.owner(), fileInfo.group(), size,
                         fi.st_mode, fileInfo.lastRead(), fileInfo.lastModified(), fileInfo.created())) {
-        //qWarning() << " prepareWriting" << destName << "failed";
+        //qCWarning(KArchiveLog) << " prepareWriting" << destName << "failed";
         return false;
     }
 
@@ -261,7 +281,7 @@ bool KArchive::addLocalFile(const QString &fileName, const QString &destName)
     qint64 total = 0;
     while ((n = file.read(array.data(), array.size())) > 0) {
         if (!writeData(array.data(), n)) {
-            //qWarning() << "writeData failed";
+            //qCWarning(KArchiveLog) << "writeData failed";
             return false;
         }
         total += n;
@@ -269,7 +289,7 @@ bool KArchive::addLocalFile(const QString &fileName, const QString &destName)
     Q_ASSERT(total == size);
 
     if (!finishWriting(size)) {
-        //qWarning() << "finishWriting failed";
+        //qCWarning(KArchiveLog) << "finishWriting failed";
         return false;
     }
     return true;
@@ -279,6 +299,9 @@ bool KArchive::addLocalDirectory(const QString &path, const QString &destName)
 {
     QDir dir(path);
     if (!dir.exists()) {
+        setErrorString(
+            tr("Directory %1 does not exist")
+            .arg(path));
         return false;
     }
     dir.setFilter(dir.filter() | QDir::Hidden);
@@ -286,7 +309,7 @@ bool KArchive::addLocalDirectory(const QString &path, const QString &destName)
     for (QStringList::ConstIterator it = files.begin(); it != files.end(); ++it) {
         if (*it != QLatin1String(".") && *it != QLatin1String("..")) {
             QString fileName = path + QLatin1Char('/') + *it;
-//            qDebug() << "storing " << fileName;
+//            qCDebug(KArchiveLog) << "storing " << fileName;
             QString dest = destName.isEmpty() ? *it : (destName + QLatin1Char('/') + *it);
             QFileInfo fileInfo(fileName);
 
@@ -308,19 +331,19 @@ bool KArchive::writeFile(const QString &name, const QByteArray &data,
 {
     const qint64 size = data.size();
     if (!prepareWriting(name, user, group, size, perm, atime, mtime, ctime)) {
-        //qWarning() << "prepareWriting failed";
+        //qCWarning(KArchiveLog) << "prepareWriting failed";
         return false;
     }
 
     // Write data
     // Note: if data is null, don't call write, it would terminate the KCompressionDevice
     if (data.constData() && size && !writeData(data.constData(), size)) {
-        //qWarning() << "writeData failed";
+        //qCWarning(KArchiveLog) << "writeData failed";
         return false;
     }
 
     if (!finishWriting(size)) {
-        //qWarning() << "finishWriting failed";
+        //qCWarning(KArchiveLog) << "finishWriting failed";
         return false;
     }
     return true;
@@ -330,6 +353,9 @@ bool KArchive::writeData(const char *data, qint64 size)
 {
     bool ok = device()->write(data, size) == size;
     if (!ok) {
+        setErrorString(
+            tr("Writing failed: %1")
+            .arg(device()->errorString()));
         d->abortWriting();
     }
     return ok;
@@ -373,6 +399,11 @@ bool KArchive::finishWriting(qint64 size)
     return doFinishWriting(size);
 }
 
+void KArchive::setErrorString(const QString &errorStr)
+{
+    d->errorStr = errorStr;
+}
+
 static QString getCurrentUserName()
 {
 #if defined(Q_OS_UNIX)
@@ -406,7 +437,7 @@ static QString getCurrentGroupName()
 KArchiveDirectory *KArchive::rootDir()
 {
     if (!d->rootDir) {
-        //qDebug() << "Making root dir ";
+        //qCDebug(KArchiveLog) << "Making root dir ";
         QString username = ::getCurrentUserName();
         QString groupname = ::getCurrentGroupName();
 
@@ -417,9 +448,9 @@ KArchiveDirectory *KArchive::rootDir()
 
 KArchiveDirectory *KArchive::findOrCreate(const QString &path)
 {
-    //qDebug() << path;
+    //qCDebug(KArchiveLog) << path;
     if (path.isEmpty() || path == QLatin1String("/") || path == QLatin1String(".")) { // root dir => found
-        //qDebug() << "returning rootdir";
+        //qCDebug(KArchiveLog) << "returning rootdir";
         return rootDir();
     }
     // Important note : for tar files containing absolute paths
@@ -432,12 +463,12 @@ KArchiveDirectory *KArchive::findOrCreate(const QString &path)
     const KArchiveEntry *ent = rootDir()->entry(path);
     if (ent) {
         if (ent->isDirectory())
-            //qDebug() << "found it";
+            //qCDebug(KArchiveLog) << "found it";
         {
             const KArchiveDirectory *dir = static_cast<const KArchiveDirectory *>(ent);
             return const_cast<KArchiveDirectory *>(dir);
         } else {
-            //qWarning() << "Found" << path << "but it's not a directory";
+            //qCWarning(KArchiveLog) << "Found" << path << "but it's not a directory";
         }
     }
 
@@ -454,7 +485,7 @@ KArchiveDirectory *KArchive::findOrCreate(const QString &path)
         parent = findOrCreate(left);   // recursive call... until we find an existing dir.
     }
 
-    //qDebug() << "found parent " << parent->name() << " adding " << dirname << " to ensure " << path;
+    //qCDebug(KArchiveLog) << "found parent " << parent->name() << " adding " << dirname << " to ensure " << path;
     // Found -> add the missing piece
     KArchiveDirectory *e = new KArchiveDirectory(this, dirname, d->rootDir->permissions(),
                                                  d->rootDir->date(), d->rootDir->user(),
@@ -503,8 +534,8 @@ void KArchivePrivate::abortWriting()
     if (saveFile) {
         saveFile->cancelWriting();
         delete saveFile;
-        saveFile = 0;
-        dev = 0;
+        saveFile = nullptr;
+        dev = nullptr;
     }
 }
 
@@ -651,7 +682,7 @@ QByteArray KArchiveFile::data() const
 {
     bool ok = archive()->device()->seek(d->pos);
     if (!ok) {
-        //qWarning() << "Failed to sync to" << d->pos << "to read" << name();
+        //qCWarning(KArchiveLog) << "Failed to sync to" << d->pos << "to read" << name();
     }
 
     // Read content
@@ -772,11 +803,11 @@ const KArchiveEntry *KArchiveDirectory::entry(const QString &_name) const
         const QString left = name.left(pos);
         const QString right = name.mid(pos + 1);
 
-        //qDebug() << "left=" << left << "right=" << right;
+        //qCDebug(KArchiveLog) << "left=" << left << "right=" << right;
 
         const KArchiveEntry *e = d->entries.value(left);
         if (!e || !e->isDirectory()) {
-            return 0;
+            return nullptr;
         }
         return static_cast<const KArchiveDirectory *>(e)->entry(right);
     }
@@ -790,18 +821,15 @@ const KArchiveFile *KArchiveDirectory::file(const QString &name) const
     if (e && e->isFile()) {
         return static_cast<const KArchiveFile *>(e);
     }
-    return 0;
+    return nullptr;
 }
 
 void KArchiveDirectory::addEntry(KArchiveEntry *entry)
 {
-    if (entry->name().isEmpty()) {
-        return;
-    }
-
     if (d->entries.value(entry->name())) {
-        /*qWarning() << "directory " << name()
+        /*qCWarning(KArchiveLog) << "directory " << name()
                     << "has entry" << entry->name() << "already";*/
+        delete entry;
         return;
     }
     d->entries.insert(entry->name(), entry);
@@ -816,12 +844,12 @@ void KArchiveDirectory::removeEntry(KArchiveEntry *entry)
     QHash<QString, KArchiveEntry *>::Iterator it = d->entries.find(entry->name());
     // nothing removed?
     if (it == d->entries.end()) {
-        qWarning() << "directory " << name()
+        qCWarning(KArchiveLog) << "directory " << name()
                    << "has no entry with name " << entry->name();
         return;
     }
     if (it.value() != entry) {
-        qWarning() << "directory " << name()
+        qCWarning(KArchiveLog) << "directory " << name()
                    << "has another entry for name " << entry->name();
         return;
     }
@@ -859,7 +887,7 @@ bool KArchiveDirectory::copyTo(const QString &dest, bool recursiveCopy) const
         // otherwise put file under root position in extraction folder
         QString curDirName = dirNameStack.pop();
         if (!QDir(curDirName).absolutePath().startsWith(destDir)) {
-            qWarning() << "Attempted export into folder" << curDirName
+            qCWarning(KArchiveLog) << "Attempted export into folder" << curDirName
                 << "which is outside of the extraction root folder" << destDir << "."
                 << "Changing export of contained files to extraction root folder.";
             curDirName = destDir;
@@ -882,7 +910,7 @@ bool KArchiveDirectory::copyTo(const QString &dest, bool recursiveCopy) const
 #endif
                 QFile symLinkTarget(curEntry->symLinkTarget());
                 if (!symLinkTarget.link(linkName)) {
-                    //qDebug() << "symlink(" << curEntry->symLinkTarget() << ',' << linkName << ") failed:" << strerror(errno);
+                    //qCDebug(KArchiveLog) << "symlink(" << curEntry->symLinkTarget() << ',' << linkName << ") failed:" << strerror(errno);
                 }
             } else {
                 if (curEntry->isFile()) {
